<p>{% if workout_map %}<img src="{{ workout_map }}" alt="{{ _('Workout map')}}"><br />{% endif %}
<strong>{{ _('Duration')}}</strong>: {{ workout_duration }}<br />
{% if workout_distance %}<strong>{{ _('Distance')}}</strong>: {{ workout_distance}} {{ workout_distance_unit }}<br />{% endif %}{% if workout_min_altitude and workout_max_altitude %}<strong>{{ _('Min. altitude')}}</strong>: {{ workout_min_altitude}} {{ workout_elevation_unit }}<br /><strong>{{ _('Max. altitude')}}</strong>: {{ workout_max_altitude}} {{ workout_elevation_unit }}<br />{% endif %}{% if workout_ascent and workout_descent %}<strong>{{ _('Ascent')}}</strong>: {{ workout_ascent }} {{ workout_elevation_unit }}<strong><br />{{ _('Descent')}}</strong>: {{ workout_descent}} {{ workout_elevation_unit }}<br />{% endif %}{% if with_description %}
<strong>{{ _('Description')}}:</strong>
{% endif %}