{% extends "layout.html" %}
{% block title %}{{ _('Email changed') }}{% endblock %}
{% block preheader %}{{ _('Your email is being updated.') }}{% endblock %}
{% block content %}<p>{{ _('You recently requested to change your email address for your FitTrackee account to:') }}</p>
                        <table class="body-action" align="center" width="100%" cellpadding="0" cellspacing="0" role="presentation">
                          <tr>
                            <td align="center">
                              <table width="100%" border="0" cellspacing="0" cellpadding="0" role="presentation">
                                <tr>
                                  <td align="center">
                                    {{new_email_address}}
                                  </td>
                                </tr>
                              </table>
                            </td>
                          </tr>
                        </table>{% endblock %}
{% block not_initiated %}{{ _("If this email change wasn't initiated by you, please change your password immediately or contact your administrator if your account is locked.") }}{% endblock %}