from flask_babel import gettext as _

# for strings detection by gettext
AVAILABLE_SPORTS = [
    _("Canoeing"),
    _("Canoeing (Whitewater)"),
    _("Cycling (Sport)"),
    _("Cycling (Transport)"),
    _("Cycling (Trekking)"),
    _("Cycling (Virtual)"),
    _("Halfbike"),
    _("Hiking"),
    _("Kayaking"),
    _("Ice Skating"),
    _("Inline Skating"),
    _("Kayaking (Whitewater)"),
    _("Mountain Biking"),
    _("Mountain Biking (Electric)"),
    _("Mountaineering"),
    _("Open Water Swimming"),
    _("Padel (Outdoor)"),
    _("Paragliding"),
    _("Rowing"),
    _("Running"),
    _("Skiing (Alpine)"),
    _("Skiing (Cross Country)"),
    _("Snowshoes"),
    _("Standup Paddleboarding"),
    _("Swimrun"),
    _("Tennis (Outdoor)"),
    _("Trail"),
    _("Walking"),
    _("Windsurfing"),
]
